from __future__ import annotations

from dataclasses import fields
from typing import Any
from typing import get_type_hints

from check_types.check_types import check_isinstance
from check_types.errors import ArgumentIsAClass
from check_types.errors import ArgumentIsNotADataClass
from check_types.errors import TypeErrorsInDataClass


def check_dataclass(
    obj: Any,
    *,
    globalns: dict[str, Any] | None = None,
    localns: dict[str, Any] | None = None,
) -> None:
    if isinstance(obj, type):
        raise ArgumentIsAClass(obj)
    try:
        obj_fields = fields(obj)
    except TypeError:
        raise ArgumentIsNotADataClass(obj)
    else:
        errors = {}
        for field in obj_fields:
            name = field.name
            value = getattr(obj, name)
            cls = get_type_hints(obj, globalns=globalns, localns=localns)[name]
            try:
                check_isinstance(value, cls)
            except TypeError as error:
                errors[name] = error
        if errors:
            raise TypeErrorsInDataClass(errors)
